#include <time.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <vga.h>

#include "tetX.h"
#include "tet_values.h"


int ShadowY;

extern int BoardX, BoardY, BoardH, BoardW;
extern int ScreenHeight, Level;
extern char  Patterns[][FIG_HEIGHT][FGB_WIDTH];
extern BOARD  Board[BRD_HEIGHT][BRD_WIDTH];
extern char  *PauseFileName;
extern char  *PlayListFileName;

extern BOTTOM_MODE BottomMode;
extern WHERE_WE_ARE where;
extern int	ShowGrid;
extern int	MusicEnabled;
extern int	JoyEX, JoyEY, JoyW, JoyH;
extern char	FillPats[][8];
extern char 	ArrowsActive[5][14][8], ArrowsPassive[5][14][8];
extern int  	ArrOffs[5][2];

static char FigImage[(FIG_DIM+2)*FIG_WIDTH];


void ShowFigure(FIGURE figure, int px, int py, int tox, int toy)
{
    int  x, y, x1, y1, x2, y2, off;
    int  patno;
    char byte;


    for (y=0; y<toy; y++)
    { y2 = y * FIG_HEIGHT;
      for (y1= 0; y1 < FIG_HEIGHT; y1++)
      {  for (x=0; x < tox; x++)
	 { x2 = x * FIG_WIDTH;
	   patno = figure[y][x];
           off =x2;
	   for (x1= 0; x1 < FGB_WIDTH; x1++)
//	      FigImage[x2+x1] = Patterns[patno][y1][x1];
	   {  byte  = Patterns[patno][y1][x1];
              FigImage[off++] = (byte >> 4) & 0xf;
              FigImage[off++] = byte & 0xf;
           }
	 }

	setrow(px, py+y2+y1, tox*FIG_WIDTH, FigImage);
      }
    }
}


/*--------------------------------------------------------*/
void ShowShadow(FIGURE figure, int pos)
{  char shadow[BRD_WIDTH], s;
   int  wid, off1;
   int  x, y, x1, fromx, tox;
   int col, row, patno;

   if (figure == NULL)
   {  RECT rc;
      rc.top = ShadowY;  rc.bottom = ShadowY + FIG_HEIGHT;
      rc.left = BoardX;  rc.right = BoardX + BoardW;
      DrawSolidRect(&rc, 0);
      return;
   }

   memset(shadow, '\0', BRD_WIDTH);

   for (col=0; col<FIG_DIM; col++)
   { x1 = pos + col;
     if (x1 >= BRD_WIDTH) break;

     s = 0;
     for (row=0; row<FIG_DIM; row++)
     { patno = figure[row][col];
       if (patno)
       { s = patno; break; }
     }

     shadow[x1] = s;

   }

   fromx = pos-1;
   if (fromx < 0)  fromx = 0;
   tox = pos + FIG_DIM +1 ;
   if (tox > BRD_WIDTH) tox = BRD_WIDTH;
//   fromx = 0;
//   tox = BRD_WIDTH;
   x = BoardX + fromx * FIG_WIDTH;
   y = ShadowY;
   wid = (tox - fromx) * FIG_WIDTH;

   memset(FigImage, '\0', sizeof(FigImage));

   for (row=0; row<FIG_HEIGHT; row++)
   {
      off1 = 0;
      for (col=fromx; col<tox; col++)
      {  patno = shadow[col];

         for (x1= 0; x1 < FGB_WIDTH; x1++)
           {  s = Patterns[patno][row][x1];
              FigImage[off1++] = (s >> 4) & 0xf;
              FigImage[off1++] = s & 0xf;
           }
      }

      setrow(x, y++, wid, FigImage);
   }
}
/*
void show_shadow_frame(void)
{   RECT rc;

    rc.top = ShadowY; rc.bottom = ShadowY + FIG_HEIGHT;
    rc.left = BoardX; rc.right = BoardX + BoardW;

    if (BottomMode == B_SHADOW && where == W_GAME)
            DrawFramedRect(&rc);
    else
    {  rc.top -= 3;  rc.bottom += 8;
       if (rc.bottom > ScreenHeight) rc.bottom = ScreenHeight;
       rc.left -=8;  rc.right += 8;
       DrawPatRect(&rc, FillPats[Level], 5-(BoardY%2));
    }
}

*/


static void show_joy_arrow(int arrowtype, char Texture[5][14][8])
{   int i, j, x, y;
    char unpacked[32], byte;
    char *from, *to;

    x =  JoyEX + ArrOffs[arrowtype][0];
    y =  JoyEY + ArrOffs[arrowtype][1];

    for (i=0; i< 14; i++)
    {  from = Texture[arrowtype][i];
       to = unpacked;
       for (j=0; j<16; j++)
       {  byte = *from++;
          *to++ = (byte >> 8) & 0xf;
          *to++ = byte & 0xf;
       }

       setrow(x, y+i, 16, unpacked);

    }
}



void  ShowJoyPos(int active)
{   int i;

    for (i=0; i<5; i++)
       show_joy_arrow(i, (i==active) ? ArrowsActive : ArrowsPassive);


}


void DrawBoardRow(char brdrow[BRD_WIDTH], int fx, int tx, int y, unsigned char mask)
{ int x, x1, y1, off, y2, x3, wid;

  char  BoardImage[BRD_WIDTH*FIG_WIDTH];
  char  byte;
  int patno;

      y2 = BoardY + y  * FIG_HEIGHT;
      x3 = BoardX + fx * FIG_WIDTH;
      wid = (tx-fx) * FIG_WIDTH;

      for (y1= 0; y1 < FIG_HEIGHT; y1++)
      {  for (x=fx; x < tx; x++)
         { off = (x-fx) * FIG_WIDTH;
           patno = brdrow[x];
           if (ShowGrid && patno == 0) patno = 8;
           for (x1= 0; x1 < FGB_WIDTH; x1++)
           {   if (patno < 12)
                  byte = Patterns[patno][y1][x1] ^ mask;
               else
                  byte = 0;
               BoardImage[off++] = (byte >> 4) & 0xf;
               BoardImage[off++] = byte & 0xf;
           }

         }
         setrow(x3, y2+y1, wid, BoardImage);
      }


     return;
}



void PauseGame(void)
{ char  buf[161];
  FILE *f, *out;

  if (!PauseFileName) return;

  SuspendTimer();
  vga_flip();

  out = fopen("/dev/console", "w+");
  if (!out) out = fopen(ttyname(fileno(stderr)), "w+");

  f = fopen (PauseFileName, "rt");

  fprintf(out, CLEAR_SCREEN);

  if (f)
  {   while(fgets(buf, sizeof(buf)-1, f))
                   fputs(buf, out);
      fclose(f);
  }
  else
  {    fprintf(out, "\n\n\n\n\n\n");
       fprintf(out, "\t\t\tPut a text, you wish\n");
       fprintf(out, "\t\t\tto be displayed here\n");
       fprintf(out, "\t\t\tin text file\n\n");
       fprintf(out, "\t\t\t%s", PauseFileName);
       fprintf(out, "\n\n\n\n\n\n\n\n\n\n\n\n\n");
       fprintf(out, "\t\t\tPress any character to resume");
  }

  while(toupper(fgetc(out))=='M' && PlayListFileName)
  {  MusicEnabled = !MusicEnabled;
     PrepareMusic(MODE_FORCE);
  }

  fprintf(out, CLEAR_SCREEN);
  fprintf(out, "\n");
  fclose(out);

  vga_flip();
  ResumeTimer();
  return;
}

